(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/admin-server.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var ADMIN_TOKEN_EXPIRATION_TIME = 15 * 60 * 1000;                      // 17
var publicAdminSettings = ["google", "github", "ldap", "emailToken", "splashUrl", "signupDialog", "adminAlert", "adminAlertTime", "adminAlertUrl", "termsUrl", "privacyUrl", "appMarketUrl", "appIndexUrl", "appUpdatesEnabled", "serverTitle", "returnAddress", "ldapNameField", "organizationEmail", "organizationLdap", "organizationGoogle"];
                                                                       //
var FEATURE_KEY_FIELDS_PUBLISHED_TO_ADMINS = ["customer", "expires", "features", "isElasticBilling", "isTrial", "issued", "userLimit"];
                                                                       //
var PUBLIC_FEATURE_KEY_FIELDS = ["expires", "features"];               // 30
                                                                       //
var Fs = Npm.require("fs");                                            // 34
var SANDSTORM_ADMIN_TOKEN = SANDSTORM_VARDIR + "/adminToken";          // 35
                                                                       //
var tokenIsValid = function (token) {                                  // 37
  if (token && Fs.existsSync(SANDSTORM_ADMIN_TOKEN)) {                 // 38
    var stats = Fs.statSync(SANDSTORM_ADMIN_TOKEN);                    // 39
    var expireTime = new Date(Date.now() - ADMIN_TOKEN_EXPIRATION_TIME);
    if (stats.mtime < expireTime) {                                    // 41
      return false;                                                    // 42
    } else {                                                           //
      return Fs.readFileSync(SANDSTORM_ADMIN_TOKEN, { encoding: "utf8" }) === token;
    }                                                                  //
  } else {                                                             //
    return false;                                                      // 47
  }                                                                    //
};                                                                     //
                                                                       //
var checkAuth = function (token) {                                     // 51
  check(token, Match.OneOf(undefined, null, String));                  // 52
  if (!isAdmin() && !tokenIsValid(token)) {                            // 53
    throw new Meteor.Error(403, "User must be admin or provide a valid token");
  }                                                                    //
};                                                                     //
                                                                       //
var clearAdminToken = function (token) {                               // 58
  if (tokenIsValid(token)) {                                           // 59
    Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                              // 60
    console.log("Admin token deleted.");                               // 61
  }                                                                    //
};                                                                     //
                                                                       //
Meteor.methods({                                                       // 65
  setAccountSetting: function (token, serviceName, value) {            // 66
    checkAuth(token);                                                  // 67
    check(serviceName, String);                                        // 68
    check(value, Boolean);                                             // 69
                                                                       //
    // TODO(someday): currently this relies on the fact that an account is tied to a single
    // identity, and thus has only that entry in "services". This will need to be looked at when
    // multiple login methods/identities are allowed for a single account.
    if (!value && !tokenIsValid(token) && serviceName in Meteor.user().services) {
      throw new Meteor.Error(403, "You can not disable the login service that your account uses.");
    }                                                                  //
                                                                       //
    // Only check configurations for OAuth services.                   //
    var oauthServices = ["google", "github"];                          // 80
    if (value && oauthServices.indexOf(serviceName) != -1) {           // 81
      var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
      var config = ServiceConfiguration.configurations.findOne({ service: serviceName });
      if (!config) {                                                   // 84
        throw new Meteor.Error(403, "You must configure the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                //
                                                                       //
      if (!config.clientId || !config.secret) {                        // 89
        throw new Meteor.Error(403, "You must provide a non-empty clientId and secret for the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                //
    }                                                                  //
                                                                       //
    Settings.upsert({ _id: serviceName }, { $set: { value: value } });
    if (value) {                                                       // 96
      Settings.update({ _id: serviceName }, { $unset: { automaticallyReset: 1 } });
    }                                                                  //
  },                                                                   //
                                                                       //
  setSmtpConfig: function (token, config) {                            // 101
    checkAuth(token);                                                  // 102
    check(config, {                                                    // 103
      hostname: String,                                                // 104
      port: Number,                                                    // 105
      auth: {                                                          // 106
        user: String,                                                  // 107
        pass: String                                                   // 108
      },                                                               //
      returnAddress: String                                            // 110
    });                                                                //
                                                                       //
    Settings.upsert({ _id: "smtpConfig" }, { $set: { value: config } });
  },                                                                   //
                                                                       //
  setSetting: function (token, name, value) {                          // 116
    checkAuth(token);                                                  // 117
    check(name, String);                                               // 118
    check(value, Match.OneOf(null, String, Date, Boolean));            // 119
                                                                       //
    Settings.upsert({ _id: name }, { $set: { value: value } });        // 121
  },                                                                   //
                                                                       //
  submitFeatureKey: function (token, textBlock) {                      // 124
    checkAuth(token);                                                  // 125
    check(textBlock, Match.OneOf(null, String));                       // 126
                                                                       //
    var db = this.connection.sandstormDb;                              // 128
                                                                       //
    if (!textBlock) {                                                  // 130
      // Delete the feature key.                                       //
      db.collections.featureKey.remove("currentFeatureKey");           // 132
      return;                                                          // 133
    }                                                                  //
                                                                       //
    // textBlock is a base64'd string, possibly with newlines and comment lines starting with "-"
    var featureKeyBase64 = _.chain(textBlock.split("\n")).filter(function (line) {
      return line.length > 0 && line[0] !== "-";                       //
    }).value().join("");                                               //
                                                                       //
    var buf = new Buffer(featureKeyBase64, "base64");                  // 142
    if (buf.length < 64) {                                             // 143
      throw new Meteor.Error(401, "Invalid feature key");              // 144
    }                                                                  //
                                                                       //
    // loadSignedFeatureKey is provided in feature-key.js              //
    var featureKey = loadSignedFeatureKey(buf);                        // 148
    if (!featureKey) {                                                 // 149
      throw new Meteor.Error(401, "Invalid feature key");              // 150
    }                                                                  //
                                                                       //
    // Persist the feature key in the database.                        //
    db.collections.featureKey.upsert("currentFeatureKey", {            // 154
      _id: "currentFeatureKey",                                        // 157
      value: buf                                                       // 158
    });                                                                //
  },                                                                   //
                                                                       //
  adminConfigureLoginService: function (token, options) {              // 163
    checkAuth(token);                                                  // 164
    check(options, Match.ObjectIncluding({ service: String }));        // 165
                                                                       //
    var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
                                                                       //
    ServiceConfiguration.configurations.upsert({ service: options.service }, options);
  },                                                                   //
                                                                       //
  clearResumeTokensForService: function (token, serviceName) {         // 172
    checkAuth(token);                                                  // 173
    check(serviceName, String);                                        // 174
                                                                       //
    var query = {};                                                    // 176
    query["services." + serviceName] = { $exists: true };              // 177
    Meteor.users.find(query).forEach(function (identity) {             // 178
      if (identity.services.resume && identity.services.resume.loginTokens && identity.services.resume.loginTokens.length > 0) {
        Meteor.users.update({ _id: identity._id }, { $set: { "services.resume.loginTokens": [] } });
      }                                                                //
                                                                       //
      Meteor.users.update({ "loginIdentities.id": identity._id }, { $set: { "services.resume.loginTokens": [] } });
    });                                                                //
  },                                                                   //
                                                                       //
  adminUpdateUser: function (token, userInfo) {                        // 189
    checkAuth(token);                                                  // 190
    check(userInfo, {                                                  // 191
      userId: String,                                                  // 192
      signupKey: Boolean,                                              // 193
      isAdmin: Boolean                                                 // 194
    });                                                                //
                                                                       //
    var userId = userInfo.userId;                                      // 197
    if (userId === Meteor.userId() && !userInfo.isAdmin) {             // 198
      throw new Meteor.Error(403, "User cannot remove admin permissions from itself.");
    }                                                                  //
                                                                       //
    Meteor.users.update({ _id: userId }, { $set: _.omit(userInfo, ["_id", "userId"]) });
  },                                                                   //
                                                                       //
  testSend: function (token, smtpConfig, to) {                         // 205
    checkAuth(token);                                                  // 206
    check(smtpConfig, {                                                // 207
      hostname: String,                                                // 208
      port: Number,                                                    // 209
      auth: {                                                          // 210
        user: String,                                                  // 211
        pass: String                                                   // 212
      },                                                               //
      returnAddress: String                                            // 214
    });                                                                //
    check(to, String);                                                 // 216
    var returnAddress = smtpConfig.returnAddress;                      //
    var restConfig = babelHelpers.objectWithoutProperties(smtpConfig, ["returnAddress"]);
                                                                       //
    SandstormEmail.send({                                              // 219
      to: to,                                                          // 220
      from: globalDb.getServerTitle() + " <" + returnAddress + ">",    // 221
      subject: "Testing your Sandstorm's SMTP setting",                // 222
      text: "Success! Your outgoing SMTP is working.",                 // 223
      smtpConfig: restConfig                                           // 224
    });                                                                //
  },                                                                   //
                                                                       //
  createSignupKey: function (token, note, quota) {                     // 228
    checkAuth(token);                                                  // 229
    check(note, String);                                               // 230
    check(quota, Match.OneOf(undefined, null, Number));                // 231
                                                                       //
    var key = Random.id();                                             // 233
    var content = { _id: key, used: false, note: note };               // 234
    if (typeof quota === "number") content.quota = quota;              // 235
    SignupKeys.insert(content);                                        // 236
    return key;                                                        // 237
  },                                                                   //
                                                                       //
  sendInvites: function (token, origin, from, list, subject, message, quota) {
    checkAuth(token);                                                  // 241
    check([origin, from, list, subject, message], [String]);           // 242
    check(quota, Match.OneOf(undefined, null, Number));                // 243
                                                                       //
    if (!from.trim()) {                                                // 245
      throw new Meteor.Error(403, "Must enter 'from' address.");       // 246
    }                                                                  //
                                                                       //
    if (!list.trim()) {                                                // 249
      throw new Meteor.Error(403, "Must enter 'to' addresses.");       // 250
    }                                                                  //
                                                                       //
    this.unblock();                                                    // 253
                                                                       //
    list = list.split("\n");                                           // 255
    for (var i in babelHelpers.sanitizeForInObject(list)) {            // 256
      var email = list[i].trim();                                      // 257
                                                                       //
      if (email) {                                                     // 259
        var key = Random.id();                                         // 260
                                                                       //
        var content = {                                                // 262
          _id: key,                                                    // 263
          used: false,                                                 // 264
          note: "E-mail invite to " + email,                           // 265
          email: email,                                                // 266
          definitelySent: false                                        // 267
        };                                                             //
        if (typeof quota === "number") content.quota = quota;          // 269
        SignupKeys.insert(content);                                    // 270
        SandstormEmail.send({                                          // 271
          to: email,                                                   // 272
          from: from,                                                  // 273
          subject: subject,                                            // 274
          text: message.replace(/\$KEY/g, origin + "/signup/" + key)   // 275
        });                                                            //
        SignupKeys.update(key, { $set: { definitelySent: true } });    // 277
      }                                                                //
    }                                                                  //
                                                                       //
    return { sent: true };                                             // 281
  },                                                                   //
                                                                       //
  offerIpNetwork: function (token) {                                   // 284
    checkAuth(token);                                                  // 285
    if (!isAdmin()) {                                                  // 286
      throw new Meteor.Error(403, "Offering IpNetwork is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
    }                                                                  //
                                                                       //
    var requirements = [{ userIsAdmin: Meteor.userId() }];             // 291
    var sturdyRef = waitPromise(saveFrontendRef({ ipNetwork: true }, { webkey: null }, requirements)).sturdyRef;
    return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
  },                                                                   //
                                                                       //
  offerIpInterface: function (token) {                                 // 300
    checkAuth(token);                                                  // 301
    if (!isAdmin()) {                                                  // 302
      throw new Meteor.Error(403, "Offering IpInterface is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
    }                                                                  //
                                                                       //
    var requirements = [{ userIsAdmin: Meteor.userId() }];             // 307
    var sturdyRef = waitPromise(saveFrontendRef({ ipInterface: true }, { webkey: null }, requirements)).sturdyRef;
    return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
  },                                                                   //
                                                                       //
  adminToggleDisableCap: function (token, capId, value) {              // 316
    checkAuth(token);                                                  // 317
    check(capId, String);                                              // 318
    check(value, Boolean);                                             // 319
                                                                       //
    if (value) {                                                       // 321
      ApiTokens.update({ _id: capId }, { $set: { revoked: true } });   // 322
    } else {                                                           //
      ApiTokens.update({ _id: capId }, { $set: { revoked: false } });  // 324
    }                                                                  //
  },                                                                   //
                                                                       //
  updateQuotas: function (token, list, quota) {                        // 328
    checkAuth(token);                                                  // 329
    check(list, String);                                               // 330
    check(quota, Match.OneOf(undefined, null, Number));                // 331
                                                                       //
    if (!list.trim()) {                                                // 333
      throw new Meteor.Error(400, "Must enter addresses.");            // 334
    }                                                                  //
                                                                       //
    var items = list.split("\n");                                      // 337
    var invalid = [];                                                  // 338
    for (var i in babelHelpers.sanitizeForInObject(items)) {           // 339
      var modifier = typeof quota === "number" ? { $set: { quota: quota } } : { $unset: { quota: "" } };
      var n = SignupKeys.update({ email: items[i] }, modifier, { multi: true });
      n += Meteor.users.update({ signupEmail: items[i] }, modifier, { multi: true });
                                                                       //
      if (n < 1) invalid.push(items[i]);                               // 345
    }                                                                  //
                                                                       //
    if (invalid.length > 0) {                                          // 348
      throw new Meteor.Error(404, "These addresses did not map to any user nor invite: " + invalid.join(", "));
    }                                                                  //
  },                                                                   //
                                                                       //
  dismissAdminStatsNotifications: function (token) {                   // 354
    checkAuth(token);                                                  // 355
    globalDb.collections.notifications.remove({ "admin.type": "reportStats" });
  },                                                                   //
                                                                       //
  signUpAsAdmin: function (token) {                                    // 359
    check(token, String);                                              // 360
    checkAuth(token);                                                  // 361
    if (!this.userId) {                                                // 362
      throw new Meteor.Error(403, "Must be logged in to sign up as admin.");
    }                                                                  //
                                                                       //
    if (!Meteor.user().loginIdentities) {                              // 366
      throw new Meteor.Error(403, "Must be logged into an account to sign up as admin.");
    }                                                                  //
                                                                       //
    Meteor.users.update({ _id: this.userId }, { $set: { isAdmin: true, signupKey: "admin" } });
    clearAdminToken(token);                                            // 371
  }                                                                    //
});                                                                    //
                                                                       //
var authorizedAsAdmin = function (token, userId) {                     // 375
  return Match.test(token, Match.OneOf(undefined, null, String)) && (userId && isAdminById(userId) || tokenIsValid(token));
};                                                                     //
                                                                       //
Meteor.publish("admin", function (token) {                             // 380
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 381
  return Settings.find();                                              // 382
});                                                                    //
                                                                       //
Meteor.publish("adminServiceConfiguration", function (token) {         // 385
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 386
  return Package["service-configuration"].ServiceConfiguration.configurations.find();
});                                                                    //
                                                                       //
Meteor.publish("publicAdminSettings", function () {                    // 390
  return Settings.find({ _id: { $in: publicAdminSettings } });         // 391
});                                                                    //
                                                                       //
Meteor.publish("adminToken", function (token) {                        // 394
  check(token, String);                                                // 395
  this.added("adminToken", "adminToken", { tokenIsValid: tokenIsValid(token) });
  this.ready();                                                        // 397
});                                                                    //
                                                                       //
Meteor.publish("allUsers", function (token) {                          // 400
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 401
  return Meteor.users.find();                                          // 402
});                                                                    //
                                                                       //
Meteor.publish("activityStats", function (token) {                     // 405
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 406
  return ActivityStats.find();                                         // 407
});                                                                    //
                                                                       //
Meteor.publish("statsTokens", function (token) {                       // 410
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 411
  return StatsTokens.find();                                           // 412
});                                                                    //
                                                                       //
Meteor.publish("allPackages", function (token) {                       // 415
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 416
  return Packages.find({ manifest: { $exists: true } }, { fields: { appId: 1, "manifest.appVersion": 1,
      "manifest.actions": 1, "manifest.appTitle": 1 } });              // 419
});                                                                    //
                                                                       //
Meteor.publish("realTimeStats", function (token) {                     // 422
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 423
                                                                       //
  // Last five minutes.                                                //
  this.added("realTimeStats", "now", computeStats(new Date(Date.now() - 5 * 60 * 1000)));
                                                                       //
  // Since last sample.                                                //
  var lastSample = ActivityStats.findOne({}, { sort: { timestamp: -1 } });
  var lastSampleTime = lastSample ? lastSample.timestamp : new Date(0);
  this.added("realTimeStats", "today", computeStats(lastSampleTime));  // 431
                                                                       //
  // TODO(someday): Update every few minutes?                          //
                                                                       //
  this.ready();                                                        // 435
});                                                                    //
                                                                       //
Meteor.publish("adminLog", function (token) {                          // 438
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 439
                                                                       //
  var logfile = SANDSTORM_LOGDIR + "/sandstorm.log";                   // 441
                                                                       //
  var fd = Fs.openSync(logfile, "r");                                  // 443
  var startSize = Fs.fstatSync(fd).size;                               // 444
                                                                       //
  // Start tailing at EOF - 8k.                                        //
  var offset = Math.max(0, startSize - 8192);                          // 447
                                                                       //
  var _this = this;                                                    // 449
  function doTail() {                                                  // 450
    for (;;) {                                                         // 451
      var buf = new Buffer(Math.max(1024, startSize - offset));        // 452
      var n = Fs.readSync(fd, buf, 0, buf.length, offset);             // 453
      if (n <= 0) break;                                               // 454
      _this.added("adminLog", offset, { text: buf.toString("utf8", 0, n) });
      offset += n;                                                     // 456
    }                                                                  //
  }                                                                    //
                                                                       //
  // Watch the file for changes.                                       //
  var watcher = Fs.watch(logfile, { persistent: false }, Meteor.bindEnvironment(doTail));
                                                                       //
  // When the subscription stops, stop watching the file.              //
  this.onStop(function () {                                            // 464
    watcher.close();                                                   // 465
    Fs.closeSync(fd);                                                  // 466
  });                                                                  //
                                                                       //
  // Read initial 8k tail data immediately.                            //
  doTail();                                                            // 470
                                                                       //
  // Notify ready.                                                     //
  this.ready();                                                        // 473
});                                                                    //
                                                                       //
Meteor.publish("adminApiTokens", function (token) {                    // 476
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 477
  return ApiTokens.find({                                              // 478
    $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
  }, {                                                                 //
    fields: {                                                          // 484
      frontendRef: 1,                                                  // 485
      created: 1,                                                      // 486
      requirements: 1,                                                 // 487
      revoked: 1                                                       // 488
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
Meteor.publish("featureKey", function (forAdmin, token) {              // 493
  var _this2 = this;                                                   //
                                                                       //
  if (forAdmin && !authorizedAsAdmin(token, this.userId)) return [];   // 494
                                                                       //
  var fields = forAdmin ? FEATURE_KEY_FIELDS_PUBLISHED_TO_ADMINS : PUBLIC_FEATURE_KEY_FIELDS;
                                                                       //
  var db = this.connection.sandstormDb;                                // 499
  var featureKeyQuery = db.collections.featureKey.find({ _id: "currentFeatureKey" });
  var observeHandle = featureKeyQuery.observe({                        // 501
    added: function (doc) {                                            // 502
      // Load and verify the signed feature key.                       //
      var buf = new Buffer(doc.value);                                 // 504
      var featureKey = loadSignedFeatureKey(buf);                      // 505
                                                                       //
      if (featureKey) {                                                // 507
        var _ref;                                                      //
                                                                       //
        // If the signature is valid, publish the feature key information.
        var filteredFeatureKey = (_ref = _).pick.apply(_ref, [featureKey].concat(fields));
        _this2.added("featureKey", doc._id, filteredFeatureKey);       // 510
      }                                                                //
    },                                                                 //
                                                                       //
    changed: function (newDoc, oldDoc) {                               // 514
      // Load and reverify the new signed feature key.                 //
      var buf = new Buffer(newDoc.value);                              // 516
      var featureKey = loadSignedFeatureKey(buf);                      // 517
                                                                       //
      if (featureKey) {                                                // 519
        var _ref2;                                                     //
                                                                       //
        // If the signature is valid, call this.changed() with the interesting fields.
        var filteredFeatureKey = (_ref2 = _).pick.apply(_ref2, [featureKey].concat(fields));
        _this2.changed("featureKey", newDoc._id, filteredFeatureKey);  // 522
      } else {                                                         //
        // Otherwise, call this.removed(), since the new feature key is invalid.
        _this2.removed("featureKey", oldDoc._id);                      // 525
      }                                                                //
    },                                                                 //
                                                                       //
    removed: function (oldDoc) {                                       // 529
      _this2.removed("featureKey", oldDoc._id);                        // 530
    }                                                                  //
  });                                                                  //
                                                                       //
  this.onStop(function () {                                            // 534
    observeHandle.stop();                                              // 535
  });                                                                  //
                                                                       //
  this.ready();                                                        // 538
});                                                                    //
                                                                       //
function observeOauthService(name) {                                   // 541
  Settings.find({ _id: name, value: true }).observe({                  // 542
    added: function () {                                               // 543
      // Tell the oauth library it should accept login attempts from this service.
      Accounts.oauth.registerService(name);                            // 545
    },                                                                 //
                                                                       //
    removed: function () {                                             // 548
      // Tell the oauth library it should deny login attempts from this service.
      Accounts.oauth.unregisterService(name);                          // 550
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
observeOauthService("github");                                         // 555
observeOauthService("google");                                         // 556
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=admin-server.js.map
