(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var ECMAScript = Package.ecmascript.ECMAScript;
var HTTP = Package.http.HTTP;
var HTTPInternals = Package.http.HTTPInternals;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;

/* Package-scope variables */
var SandstormAutoupdateApps;

(function(){

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                               //
// packages/sandstorm-autoupdate-apps/autoupdate-apps.js                                                         //
//                                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                                 //
// Sandstorm - Personal Cloud Sandbox                                                                            //
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                                         //
// All rights reserved.                                                                                          //
//                                                                                                               //
// Licensed under the Apache License, Version 2.0 (the "License");                                               //
// you may not use this file except in compliance with the License.                                              //
// You may obtain a copy of the License at                                                                       //
//                                                                                                               //
//   http://www.apache.org/licenses/LICENSE-2.0                                                                  //
//                                                                                                               //
// Unless required by applicable law or agreed to in writing, software                                           //
// distributed under the License is distributed on an "AS IS" BASIS,                                             //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                                      //
// See the License for the specific language governing permissions and                                           //
// limitations under the License.                                                                                //
                                                                                                                 //
SandstormAutoupdateApps = {};                                                                                    // 17
                                                                                                                 //
SandstormAutoupdateApps.updateAppIndex = function (db) {                                                         // 19
  var appUpdatesEnabledSetting = db.collections.settings.findOne({ _id: "appUpdatesEnabled" });                  // 20
  var appUpdatesEnabled = appUpdatesEnabledSetting && appUpdatesEnabledSetting.value;                            // 21
  if (!appUpdatesEnabled) {                                                                                      // 22
    // It's much simpler to check appUpdatesEnabled here rather than reactively deactivate the                   //
    // timer that triggers this call.                                                                            //
    return;                                                                                                      // 25
  }                                                                                                              //
                                                                                                                 //
  var appIndexUrl = db.collections.settings.findOne({ _id: "appIndexUrl" }).value;                               // 28
  var appIndex = db.collections.appIndex;                                                                        // 29
  var data = HTTP.get(appIndexUrl + "/apps/index.json").data;                                                    // 30
  data.apps.forEach(function (app) {                                                                             // 31
    app._id = app.appId;                                                                                         // 32
                                                                                                                 //
    var oldApp = appIndex.findOne({ _id: app.appId });                                                           // 34
    app.hasSentNotifications = false;                                                                            // 35
    appIndex.upsert({ _id: app._id }, app);                                                                      // 36
    if ((!oldApp || app.versionNumber > oldApp.versionNumber) && db.collections.userActions.findOne({ appId: app.appId })) {
      var pack = db.collections.packages.findOne({ _id: app.packageId });                                        // 39
      var url = appIndexUrl + "/packages/" + app.packageId;                                                      // 40
      if (pack) {                                                                                                // 41
        if (pack.status === "ready") {                                                                           // 42
          if (pack.appId && pack.appId !== app.appId) {                                                          // 43
            console.error("app index returned app ID and package ID that don't match:", JSON.stringify(app));    // 44
          } else {                                                                                               //
            db.sendAppUpdateNotifications(app.appId, app.packageId, app.name, app.versionNumber, app.version);   // 47
          }                                                                                                      //
        } else {                                                                                                 //
          var newPack = Packages.findAndModify({                                                                 // 51
            query: { _id: app.packageId },                                                                       // 52
            update: { $set: { isAutoUpdated: true } }                                                            // 53
          });                                                                                                    //
          if (newPack.status === "ready") {                                                                      // 55
            // The package was marked as ready before we applied isAutoUpdated=true. We should send              //
            // notifications ourselves to be sure there's no timing issue (sending more than one is              //
            // fine, since it will de-dupe).                                                                     //
            if (pack.appId && pack.appId !== app.appId) {                                                        // 59
              console.error("app index returned app ID and package ID that don't match:", JSON.stringify(app));  // 60
            } else {                                                                                             //
              db.sendAppUpdateNotifications(app.appId, app.packageId, app.name, app.versionNumber, app.version);
            }                                                                                                    //
          } else if (newPack.status === "failed") {                                                              //
            // If the package has failed, retry it                                                               //
            db.startInstall(app.packageId, url, true, true);                                                     // 68
          }                                                                                                      //
        }                                                                                                        //
      } else {                                                                                                   //
        db.startInstall(app.packageId, url, false, true);                                                        // 72
      }                                                                                                          //
    }                                                                                                            //
  });                                                                                                            //
};                                                                                                               //
                                                                                                                 //
Meteor.methods({                                                                                                 // 78
  updateApps: function (appUpdates) {                                                                            // 79
    var db = this.connection.sandstormDb;                                                                        // 80
    var backend = this.connection.sandstormBackend;                                                              // 81
                                                                                                                 //
    _.forEach(appUpdates, function (val, appId) {                                                                // 83
      var pack = db.collections.packages.findOne({ _id: val.packageId, appId: appId });                          // 84
      if (!pack || !pack.manifest) {                                                                             // 85
        console.error("Newer app not installed", val.name);                                                      // 86
      } else {                                                                                                   //
        db.addUserActions(val.packageId);                                                                        // 88
        db.upgradeGrains(appId, val.version, val.packageId, backend);                                            // 89
        Meteor.call("deleteUnusedPackages", appId);                                                              // 90
      }                                                                                                          //
    });                                                                                                          //
  }                                                                                                              //
});                                                                                                              //
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-autoupdate-apps'] = {
  SandstormAutoupdateApps: SandstormAutoupdateApps
};

})();

//# sourceMappingURL=sandstorm-autoupdate-apps.js.map
